/**
 * Created by u067265 on 2/21/17.
 */
/**
 * Created by elydelacruz on 2/6/16.
 */

'use strict';

let path =      require('path'),
    crypto =    require('crypto'),
    gulp =      require('gulp'),
    eslint =    require('gulp-eslint'),
    concat =    require('gulp-concat'),
    htmlmin =   require('gulp-htmlmin'),
    uglify =    require('gulp-uglify'),
    sass =      require('gulp-sass'),
    header =    require('gulp-header'),
    fncallback = require('gulp-fncallback'),
    sourcemaps = require('gulp-sourcemaps'),
    gulpIf =    require('gulp-if'),
    del =       require('del'),
    chalk =     require('chalk'),
    log = console.log.bind(console),
    packageJson =   require('./package.json'),
    gulpConfig =   require('./gulpfileConfig.json'),

    // Common paths
    // Build path
    // publicDevPath = './public-dev',
    publicPath = './public',

    fileHeaderBody = '<%= fileName %> <%= version %> | License: <%= license %> | Homepage: <%= homepage %> | ' +
        '<%= hashAlgo.toUpperCase() %> Checksum: <%= fileHash %> | Generated: <%= date %>',

    minifiedFileHeader = '/**! ' + fileHeaderBody + ' **/',

    minifiedHtmlHeader = '<!--! ' + fileHeaderBody + ' -->',

    argv = require('yargs')
        .default('dev', false)
        .argv,
    devMode = argv.dev;

function clean () {
    const paths = gulpConfig.clean.srcArgs[0];
    return del(paths)
        .then(paths => {
            if (paths.length > 0) {
                log(chalk.dim('\nThe following file paths have been deleted: \n - ' + paths.join('\n - ') + '\n'));
            }
            else {
                log(chalk.dim(' - No paths to clean.') + '\n');
            }
        })
        .catch(log);
}

function uglifyJs (srcArgs, dest) {
    return gulp.src.apply(gulp, srcArgs)
        .pipe(sourcemaps.init())
        .pipe(gulpIf(!devMode, uglify()))
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(dest))

}

function copyVendorJs () {
    const config = gulpConfig.copyVendorsJs;
    return gulp.src.apply(gulp, config.srcArgs)
        .pipe(gulp.dest(config.dest));
}

function buildVendorJs () {
    const config = gulpConfig.vendorJs;
    return uglifyJs(config.srcArgs, config.dest);
}

function buildHtml () {
    const hashAlgo = 'sha1',
        data = {hashAlgo: hashAlgo, date: new Date()};
    return gulp.src.apply(gulp, gulpConfig.html.srcArgs)
        .pipe(htmlmin({
            removeComments: true,
            collapseWhitespace: true
        }))
        .pipe(fncallback(function (file, enc, cb) {
            let hasher = crypto.createHash(hashAlgo);
            hasher.update(file.contents.toString(enc));
            data.fileHash = hasher.digest('hex');
            data.fileName = path.basename(file.path);
            cb();
        }))
        .pipe(header(minifiedHtmlHeader, Object.assign(data, packageJson)))
        .pipe(gulp.dest(publicPath));
}

function buildCss () {
    const hashAlgo = 'sha1',
        data = {hashAlgo: hashAlgo, date: new Date()};
    return gulp.src(gulpConfig.sass.srcArgs[0])
        .pipe(sass({
            outputStyle: devMode ? 'expanded' : 'compressed'
        }).on('error', sass.logError))
        .pipe(fncallback(function (file, enc, cb) {
            let hasher = crypto.createHash(hashAlgo);
            hasher.update(file.contents.toString(enc));
            data.fileHash = hasher.digest('hex');
            data.fileName = path.basename(file.path);
            cb();
        }))
        .pipe(header(minifiedFileHeader, Object.assign(data, packageJson)))
        .pipe(gulp.dest(publicPath + '/css'));
}

function buildJs () {
    const hashAlgo = 'sha1',
        data = {hashAlgo: hashAlgo, date: new Date()};
    return gulp.src.apply(gulp, gulpConfig.js.srcArgs)
        .pipe(sourcemaps.init())
        .pipe(gulpIf(devMode, uglify()))
        .pipe(fncallback(function (file, enc, cb) {
            let hasher = crypto.createHash(hashAlgo);
            hasher.update(file.contents.toString(enc));
            data.fileHash = hasher.digest('hex');
            data.fileName = path.basename(file.path);
            cb();
        }))
        .pipe(header(minifiedFileHeader, Object.assign(data, packageJson)))
        .pipe(sourcemaps.write())
        .pipe(gulp.dest(publicPath));
}

gulp.task('clean', clean);

gulp.task('copy-vendor-js', copyVendorJs);

gulp.task('copy', ['copy-vendor-js']);

gulp.task('build-vendor-js', ['copy-vendor-js'], buildJs);

gulp.task('build-js', ['build-vendor-js'], buildJs);

gulp.task('build-html', buildHtml);

gulp.task('build-css', buildCss);

gulp.task('build', ['copy', 'build-html', 'build-css', 'build-js']);

gulp.task('watch', ['build'], function () {
    gulp.watch('./public-dev/**/*.html', ['build-html']);
    gulp.watch('./public-dev/sass/**/*.scss', ['build-css']);
    gulp.watch('./public-dev/**/*.@(json|js)', ['build-js']);
});

gulp.task('default', ['watch'], function () {
    if (devMode) {
        log('\nRunning in "--dev" mode.\n');
    }
    return Promise.resolve();
});
